/**
*	project2.cpp
*	CS 6610 - Advanced Computer Graphics I
*	Zach Gildersleeve
*	Oct. 13, 2006
*	
*	Creates a square room with several objects that demonstrate
*	different aspects of texture mapping.  The room walls and floor
*	are textured with multitextured texture objects.  The cone
*	demonstrates different texture environment modes, and the sphere
*	is mapped with a dynamic reflective cube map environment.
*	Additionally, a dragonfly model incorporates other texture elements.
*	Detailed documentation for this project can be found in "REPORT.html"
*/

#include "main.h"
#include "vector.h"

int cur_button = -1;				// mouse handling for main window
int last_x;							// mouse position
int last_y;							// mouse position

GLUI				*glui;			// pointers for all of the glui controls
GLUI_Rollout		*coneRollout;
GLUI_Rollout		*lightRollout;
GLUI_RadioGroup		*coneTextureRadio;
GLUI_RadioGroup		*roomTextureRadio;
GLUI_RadioGroup		*roomEnvironmentRadio;
GLUI_Translation	*lightmapTrans;

int main_window;					// the main window name
float eye[3];						// camera position
float lookat[3];					// camera look position
float coneRotation[16];				// GLUI rotation arcball matrix
float lightmapPosition[2];			// floor lightmap position

float lightIntensity = 1.0;			// controls intensity of scene light
int coneMode = 0;					// controls cone texture environment mode
int roomTextureMode = 1;			// controls room texture on/off
int roomEnv = 0;					// controls room texture environment

GLfloat	 lightPos[]		= {25.0, 25.0, 5.0, 1.0};	// position of light
GLfloat  diffuseLight[] = {1.0, 1.0, 1.0, 0.0};		// color of light

static unsigned *image;				// pointer to texture data
static int width;					// width of texture data
static int height;					// height of texture data
static int components;				// components of texture data

GLfloat rotate = 300.0;				// controls rotation of cone,
									// defaults at a visually nice starting point

GLuint cubeMap;						// pointer to cube map
static GLuint cubeFaces[6] = {		// OpenGL implementation of cube map faces
	GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,
    GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,
    GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,
    GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB};

GLuint texName[7];					// array of non-cube map textures

/**
*	Idle function for GLUT callback
*/
void myGlutIdle(void)
{
	if (glutGetWindow() != main_window)
		glutSetWindow(main_window);
	glutPostRedisplay();
}

/**
*	Mouse callback function for GLUT
*
*	@param	button	the current mouse button
*	@param	state	the state of 'button'
*	@param	x		the x coord of the mouse
*	@param	y		the y coord of the mouse
*/
void myGlutMouse(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)
		cur_button = button;
	else
	{
		if (button == cur_button)
			cur_button = -1;
	}
	
	last_x = x;
	last_y = y;
}

/**
*	Mouse motion callback function for GLUT
*
*	@param	x		the x coord of the mouse
*	@param	y		the y coord of the mouse
*/
void myGlutMotion(int x, int y)
{
	int dx = x-last_x;			// change in mouse position
	int dy = y-last_y;			// change in mouse position
	
	float scale, len, theta;	// controls camera movement
	float neye[3], neye2[3];	// temp eye positions
	float f[3], r[3], u[3];		// more camera coords
	
	switch(cur_button)
	{
		case GLUT_LEFT_BUTTON:	// translate
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			
			scale = sqrt(v_getVectorLength(f)) * 0.007;	// scale the change by how far away we are
			
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			v_normalize(r);
			v_normalize(u);
			
			eye[0] += -r[0]*dx*scale + u[0]*dy*scale;
			eye[1] += -r[1]*dx*scale + u[1]*dy*scale;
			eye[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			lookat[0] += -r[0]*dx*scale + u[0]*dy*scale;
			lookat[1] += -r[1]*dx*scale + u[1]*dy*scale;
			lookat[2] += -r[2]*dx*scale + u[2]*dy*scale;
			
			break;
			
		case GLUT_MIDDLE_BUTTON:	// zoom
			f[0] = lookat[0] - eye[0];
			f[1] = lookat[1] - eye[1];
			f[2] = lookat[2] - eye[2];
			
			len = v_getVectorLength(f);
			v_normalize(f);
			
			len -= sqrt(len)*dx*0.03;	// scale the change by how far away we are
			
			eye[0] = lookat[0] - len*f[0];
			eye[1] = lookat[1] - len*f[1];
			eye[2] = lookat[2] - len*f[2];
			
			// make sure the eye and lookat points are sufficiently far away
			// push the lookat point forward if it is too close
			if (len < 1)
			{
				lookat[0] = eye[0] + f[0];
				lookat[1] = eye[1] + f[1];
				lookat[2] = eye[2] + f[2];
			}
			break;
			
		case GLUT_RIGHT_BUTTON:	// rotate
			
			neye[0] = eye[0] - lookat[0];
			neye[1] = eye[1] - lookat[1];
			neye[2] = eye[2] - lookat[2];
			
			// first rotate in the x/z plane
			theta = -dx * 0.007;
			neye2[0] = (float)cos(theta)*neye[0] + (float)sin(theta)*neye[2];
			neye2[1] = neye[1];
			neye2[2] =-(float)sin(theta)*neye[0] + (float)cos(theta)*neye[2];
			
			// now rotate vertically
			theta = -dy * 0.007;
			
			f[0] = -neye2[0];
			f[1] = -neye2[1];
			f[2] = -neye2[2];
			u[0] = 0;
			u[1] = 1;
			u[2] = 0;
			v_crossProduct(f, u, r);
			v_crossProduct(r, f, u);
			len = v_getVectorLength(f);
			v_normalize(f);
			v_normalize(u);
			
			neye[0] = len * ((float)cos(theta)*f[0] + (float)sin(theta)*u[0]);
			neye[1] = len * ((float)cos(theta)*f[1] + (float)sin(theta)*u[1]);
			neye[2] = len * ((float)cos(theta)*f[2] + (float)sin(theta)*u[2]);
			
			eye[0] = lookat[0] - neye[0];
			eye[1] = lookat[1] - neye[1];
			eye[2] = lookat[2] - neye[2];
			
			break;
	}
	
	last_x = x;
	last_y = y;
	
	glutPostRedisplay();
}

/**
*	Switches between texture environment mode
*	for cone.
*
*	@param	mode	current cone texture mode selected
*/
void textureMode(int mode)
{
	GLfloat red[] = {1.0, 0.0, 0.0, 0.0};	// blend color
	
	switch (mode)
	{
		case 0:
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
			break;
		case 1:
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			break;			
		case 2:
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			break;		
		case 3:
			glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, red);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND);
			break;
	}
	glutPostRedisplay();
}

/**
*	Switches material attributes, controls object
*	color when lighting is enabled.
*
*	@param	mode	current material object selected
*/
void materialMode(int mode)
{
	GLfloat none[] = {0.0, 0.0, 0.0, 1.0};
	GLfloat gray[] = {0.5, 0.5, 0.5, 1.0};
	GLfloat cyan[] = {0.0, 1.5, 1.5, 1.0};
	GLfloat white[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat coneEmission[] = {0.3, 0.3, 0.3, 0.0};
	GLfloat specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat medShiny[] = {50.0};
	GLfloat highShiny[] = {150.0};
	
	switch(mode)
	{
		case FLOOR_OBJECT:
		case ROOM_OBJECT:
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, gray);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, none);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, none);
			break;
		case CONE_OBJECT:
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, white);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, none);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, highShiny);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, coneEmission);
			break;
		case SPHERE_OBJECT:
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, gray);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, medShiny);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, none);	
			break;
		case EYE_OBJECT:
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, cyan);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, highShiny);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, none);
			break;
		case LEG_OBJECT:
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, none);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, highShiny);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, none);
			break;		
	}
}

/**
*	Initializes the OpenGL state, builds a light,
*	and loads all the textures.  Also loads temp
*	textures for cubemap
*/
GLvoid Init(GLvoid)
{
	GLfloat  ambientLight[] = {0.0, 0.0, 0.0, 0.0};		// color of overall ambient light
	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_BLEND);
	glShadeModel(GL_SMOOTH);
	
	glEnable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambientLight);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glEnable(GL_LIGHT0);

	glClearColor(0.0, 0.0, 0.0, 0.0);
	
	glGenTextures(7, texName);		// standard texture objects
	glBindTexture(GL_TEXTURE_2D, texName[0]);
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/plank.rgb", &width, &height, &components);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, width, height, GL_RGBA, GL_UNSIGNED_BYTE, image);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 8);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, texName[1]);
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/eoe3.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_MIRRORED_REPEAT);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);			
	glBindTexture(GL_TEXTURE_2D, texName[2]);	
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/light.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, texName[3]);	
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/wing.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);
	glBindTexture(GL_TEXTURE_2D, texName[4]);	
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/fresco.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);							
	glBindTexture(GL_TEXTURE_2D, texName[5]);	
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/wall.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);	
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);																					
	glBindTexture(GL_TEXTURE_2D, texName[6]);	
		image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/dragonfly.rgb", &width, &height, &components);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_REPEAT);		
		glTexImage2D(GL_TEXTURE_2D, 0, components, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		free(image);																																			

	glEnable(GL_TEXTURE_CUBE_MAP_ARB);	
	glGenTextures(1, &cubeMap);			// cubemap texture objects
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubeMap);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_R, GL_MIRRORED_REPEAT);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);
	   
		for(int i = 0; i < 6; i++)
		{
			image = read_texture("/Users/zachgildersleeve/Documents/Soc\ F2006/CP\ SC\ 6610\ Computer\ Graphics/project2/project2/test.rgb", &width, &height, &components);

			glTexImage2D(cubeFaces[i], 0, GL_RGBA, CUBEMAP_SIZE, CUBEMAP_SIZE, 0, GL_RGBA, GL_UNSIGNED_BYTE, image);
		}
		glDisable(GL_TEXTURE_CUBE_MAP_ARB);
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);

	glActiveTexture(GL_TEXTURE0);		// makes texture unit for multitexturing
		glBindTexture(GL_TEXTURE_2D, texName[0]);

	glActiveTexture(GL_TEXTURE2);		// makes texture unit for multitexturing
		glBindTexture(GL_TEXTURE_2D, texName[2]);
}

/**
*	Reshape callback function for GLUT
*
*	@param	x	the x window coord - unused
*	@param	y	the y window coord - unused
*/
void myGlutReshape(int	x, int y)
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area(&tx, &ty, &tw, &th);
	glViewport(tx, ty, tw, th);
	
//	glutPostRedisplay();
}

/**
*	Draws the reflecting sphere, and textures it with the
*	cubemap.  Attempts to rotate cubemap texture according
*	to the camera position, which is not exact when the dot product = -1.0.
*/
void DrawSphere(void)
{
	float eyeVector[] = {eye[0], eye[1], eye[2]};	// temp camera position
	float view[] = {0.0, 0.0, -1.0};				// view position
	float upVector[3];								// temp up vector
	GLfloat dotProduct;
	GLfloat angle;
	
	v_normalize(eyeVector);
	v_crossProduct(eyeVector, view, upVector);
	dotProduct = v_dotProduct(eyeVector, view);
	angle = 57.3 * acos(dotProduct);				// cheap way of converting to degrees

	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR);
	
	glDisable(GL_TEXTURE_2D);	
	glEnable(GL_TEXTURE_CUBE_MAP_ARB);	
		glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubeMap);
		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_GEN_R);
	
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
		glLoadIdentity();
		glRotatef(180.0, 0.0, 1.0, 0.0);
		glRotatef(angle, upVector[0], upVector[1], upVector[2]);
		materialMode(SPHERE_OBJECT);		
		glutSolidSphere(5.0, 200.0, 200.0);
	glPopMatrix();
	
	glMatrixMode(GL_MODELVIEW);
	
	glDisable(GL_TEXTURE_CUBE_MAP_ARB);	
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_R);
		glDisable(GL_TEXTURE_2D);
		
	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SINGLE_COLOR);	

	glEnable(GL_LIGHTING);
}

/**
*	Draws the cone, and textures the cone
*	using different texture environment modes.
*/
GLvoid drawCone (GLvoid)
{
	GLfloat tex = 0.0;						// texture coordinate counter
	GLfloat coneTop[] = {0.0, 1.0, 0.0};	// location of cone 'point'
	GLfloat cone2[] = {0.0, 0.0, 0.0};		// location of cone vertex
	GLfloat cone3[] = {0.0, 0.0, 0.0};		// location of cone vertex
	GLfloat normal[3];						// normal of cone vertex
	
	glPushMatrix();
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_SRC_COLOR);
		materialMode(CONE_OBJECT);
		glDisable(GL_LIGHTING);
		
		glActiveTexture(GL_TEXTURE0);
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, texName[1]);

		glTranslatef(0.0, -20.0, 0.0);
		glScalef(5.0, 15.0, 5.0);
		glRotatef(rotate, 0.0, 1.0, 0.0);
		textureMode(coneMode);

		glBegin(GL_TRIANGLE_FAN);
			glTexCoord2f(0.25, 1.0);
			glColor4f(1.0, 1.0, 1.0, 0.5);
			glVertex3fv(coneTop);

			for (GLfloat angle = 0.0; angle < (2.0 * GL_PI); angle += (GL_PI/8.0))
			{
				if (tex > 1.0)
				{
					tex = 0.0;
				}
			
				glTexCoord2f(tex, 0.0);
				
				cone3[0] = cone2[0];
				cone3[2] = cone2[2];
				cone2[0] = sin(angle);
				cone2[2] = cos(angle);

				v_getNormal(coneTop, cone2, cone3, normal);
				glNormal3fv(normal);

				glVertex3fv(cone2);
				tex += (1.0 / 8.0);
			}

			glTexCoord2f(tex, 0.0);
			cone2[0] = sin(0.0);
			cone2[2] = cos(0.0);
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3fv(cone2);
		glEnd();
		
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_LIGHTING);	
	glPopMatrix();	
}

/**
*	Draws a wing for the dragonfly, and textures it.
*/
void drawWing(void)
{
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_SRC_COLOR);
	glDisable(GL_LIGHTING);
	
	glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texName[3]);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);	
		
	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);	glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(1.0, 0.0);	glVertex3f(8.0, 0.0, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(8.0, 0.0, 4.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, 0.0, 4.0);
	glEnd();
	
	glEnable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

/**
*	Draws a leg for the dragonfly.
*/
void drawLeg(void)
{
	materialMode(LEG_OBJECT);
	glPushMatrix();
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glRotatef(-60.0, 1.0, 0.0, 0.0);
		glutSolidCone(0.3, 3.0, 5, 5);
		glutSolidSphere(0.4, 5.0, 5.0);
	glPopMatrix();
	
	glPushMatrix();
		glRotatef(90.0, 0.0, 1.0, 0.0);
		glRotatef(45.0, 1.0, 0.0, 0.0);
		glutSolidCone(0.3, 2.0, 5, 5);
		glTranslatef(0.0, 0.0, 2.0);
		glutSolidSphere(0.2, 2.0, 2.0);
	glPopMatrix();
}

/**
*	Draws a sphere, and includes texture coords that can be changed,
*	compared to static texture coords available with glutSolidSphere().
*	Inspired by a algorithm provided by Paul Bourke on his website at
*   http://local.wasp.uwa.edu.au/~pbourke/ .  As this was an extra
*	element beyond the project scope, the functionality of the texture
*	placement provided is limited.  This is discussed in the project
*	report associated with this project.
*
*	@param	r	the radius of the sphere
*	@param	s1	the starting 's' texture coord
*	@param	t1	the starting 't' texture coord
*	@param	s2	the ending 's' texture coord
*	@param	t2	the ending 't' texture coord
*/
void CreateSphere(float r, float s1, float t1, float s2, float t2)
{
	int i,j;
	float theta1,theta2,theta3;
	float e[3],p[3];
	float n = 20.0;
	
	for (j=0;j<n/2;j++) {
		theta1 = j * TWOPI / n - PID2;
		theta2 = (j + 1) * TWOPI / n - PID2;
		
		glBegin(GL_QUAD_STRIP);
		for (i=0;i<=n;i++) {
			theta3 = i * TWOPI / n;
			
			e[0] = cos(theta2) * cos(theta3);
			e[1] = sin(theta2);
			e[2] = cos(theta2) * sin(theta3);
			p[0] = r * e[0];
			p[1] = r * e[1];
			p[2] = r * e[2];
			
			glNormal3fv(e);
			glTexCoord2f((s1 + i/n) * s2, (t1 + 2*(j+1)/n) * t2);
			glVertex3fv(p);
			
			e[0] = cos(theta1) * cos(theta3);
			e[1] = sin(theta1);
			e[2] = cos(theta1) * sin(theta3);
			p[0] = r * e[0];
			p[1] = r * e[1];
			p[2] = r * e[2];
			
			glNormal3fv(e);
			glTexCoord2f((s1 + i/n) * s2, (t1 + 2*j/n) * t2);
			glVertex3fv(p);
		}
		glEnd();
	}
}

/**
*	Draws the dragonfly body, and textures it appropriatly.
*	The dragonfly eyes make use of the current (cone) cubemap texture
*/
void drawDragonfly(void)
{
	glPushMatrix();
		glEnable(GL_TEXTURE_2D);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
		glBindTexture(GL_TEXTURE_2D, texName[6]);
		glTranslatef(0.0, 15.0, 0.0);

		glPushMatrix();		// draws tail
			for (int i = 0; i < 5; i++)
			{
				glPushMatrix();
				glRotatef(90.0, 1.0, 0.0, 0.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
					glScalef(1.0, 2.0, 1.0);
					CreateSphere(0.5, 0.0, 0.0, 0.5, 0.5);
				glPopMatrix();
				glTranslatef(0.0, 0.0, 1.5);	
			}
		glPopMatrix();
		
		glPushMatrix();		// draws end of tail
			glRotatef(90.0, 1.0, 0.0, 0.0);
			glRotatef(90.0, 0.0, 1.0, 0.0);
			glTranslatef(0.0, 7.5, 0.0);
			CreateSphere(1.0, 0.0, 0.0, 0.5, 0.5);
		glPopMatrix();
		
		glPushMatrix();		// draws body
		glRotatef(90.0, 1.0, 0.0, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);		
			glScalef(1.5, 2.0, 1.5);
			glTranslatef(0.4, -1.2, 0.0);
					glMatrixMode(GL_TEXTURE);
					glPushMatrix();
					glLoadIdentity();
					glRotatef(-90.0, 0.0, 0.0, 1.0);
			CreateSphere(1.0, 0.0, 0.0, 0.5, 0.5);
			glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
		
		glPushMatrix();		// draws head
		glPushMatrix();
		glRotatef(90.0, 1.0, 0.0, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);		
		
			glTranslatef(0.0, -4.0, 0.0);
			CreateSphere(1.0, 0.0, 0.0, 0.5, 0.5);
		glPopMatrix();
			glTranslatef(0.0, 0.0, -4.0);
			glTranslatef(0.0, 0.2, -1.3);
			glMatrixMode(GL_TEXTURE);
			glRotatef(-90.0, 0.0, 0.0, 1.0);
			CreateSphere(1.2, 0.0, 0.0, 0.5, 0.5);

			glMatrixMode(GL_TEXTURE);
			glPopMatrix();
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();		// draws left eye
				glTranslatef(0.8, 0.0, -0.3);
				glRotatef(30.0, 0.0, 1.0, 0.0);
				glScalef(0.5, 1.0, 0.8);

				materialMode(EYE_OBJECT);
				glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SEPARATE_SPECULAR_COLOR);
				glDisable(GL_TEXTURE_2D);	
				glEnable(GL_TEXTURE_CUBE_MAP_ARB);	
					glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubeMap);
					glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
				glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_ARB);
				
				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T);
				glEnable(GL_TEXTURE_GEN_R);

				glutSolidSphere(1.0, 20.0, 20.0);
			glPopMatrix();
			
			glPushMatrix();		// draws right eye
				glTranslatef(-0.8, 0.0, -0.3);
				glRotatef(-30.0, 0.0, 1.0, 0.0);
				glScalef(0.5, 1.0, 0.8);

				glutSolidSphere(1.0, 20.0, 20.0);

				glDisable(GL_TEXTURE_GEN_S);
				glDisable(GL_TEXTURE_GEN_T);
				glDisable(GL_TEXTURE_GEN_R);
				glDisable(GL_TEXTURE_2D);
				
				glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL,GL_SINGLE_COLOR);	
				glDisable(GL_TEXTURE_CUBE_MAP_ARB);
				
				glEnable(GL_LIGHTING);
			glPopMatrix();
						
			glPushMatrix();
				glTranslatef(-1.5, -2.0, 2.0);
				drawLeg();
				glScalef(1.3, 1.3, 1.3);
				glTranslatef(0.0, 0.0, 0.8);
				glRotatef(-15.0, 1.0, 0.0, 0.0);
				drawLeg();
				glScalef(0.7, 0.7, 0.7);
				glTranslatef(0.0, 0.0, 0.9);
				drawLeg();
			glPopMatrix();

			glPushMatrix();
				glRotatef(180.0, 0.0, 1.0, 0.0);
				glTranslatef(-1.5, -2.0, -2.0);
				drawLeg();
				glScalef(1.3, 1.3, 1.3);
				glTranslatef(0.0, 0.0, -0.8);
				glRotatef(-15.0, 1.0, 0.0, 0.0);
				drawLeg();
				glScalef(0.7, 0.7, 0.7);
				glTranslatef(0.0, 0.0, -0.9);
				drawLeg();
			glPopMatrix();
			glPushMatrix();	// draws wings
			glTranslatef(0.0, 0.9, 0.5);
			glRotatef(20.0, 0.0, 0.0, 1.0);
			drawWing();
			glRotatef(-40.0, 0.0, 0.0, 1.0);
			glTranslatef(0.0, 0.0, 0.3);
			glScalef(1.2, 1.2, 1.9);
			drawWing();
			glPopMatrix();
			
			glPushMatrix();
			glRotatef(180.0, 0.0, 0.0, 1.0);
			glTranslatef(0.0, -0.9, 0.5);
			glRotatef(-20.0, 0.0, 0.0, 1.0);
			drawWing();
			glRotatef(40.0, 0.0, 0.0, 1.0);
			glTranslatef(0.0, 0.0, 0.3);
			glScalef(1.2, 1.2, 1.9);
			drawWing();
			glPopMatrix();
		glPopMatrix();
	glPopMatrix();
}

/**
*	Draws the room, and textures the walls, floor, and ceiling.
*	Adds the cone and dragonfly to the room.
*/
void DrawRoom(void)
{
	glPushMatrix();
		glScalef(20.0, 20.0, 20.0);
		
		glActiveTexture(GL_TEXTURE0);
			glEnable(GL_TEXTURE_2D);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
			glBindTexture(GL_TEXTURE_2D, texName[0]);
		glActiveTexture(GL_TEXTURE2);
			glEnable(GL_TEXTURE_2D);
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);		

		// binds lightmap to interior of floor, so light doesn't
		// go 'under' walls too much
		if(lightmapPosition[0] > 0.22) 
			lightmapPosition[0] = 0.22;
		if(lightmapPosition[0] < -0.22)
			lightmapPosition[0] = -0.22;
		if(lightmapPosition[1] > 0.22) 
			lightmapPosition[1] = 0.22;
		if(lightmapPosition[1] < -0.22)
			lightmapPosition[1] = -0.22;
			
		materialMode(FLOOR_OBJECT);	
			
		glBegin(GL_QUADS);
			glNormal3d(0, -1, 0);
			
			glMultiTexCoord2f(GL_TEXTURE0, 1.0, 0.0);
			glMultiTexCoord2f(GL_TEXTURE2, lightmapPosition[0] + 0.7, lightmapPosition[1] + 0.3);
			glVertex3f( 1.0f,-1.0f, 1.0f);
			
			glMultiTexCoord2f(GL_TEXTURE0, 0.0, 0.0);
			glMultiTexCoord2f(GL_TEXTURE2, lightmapPosition[0] + 0.3, lightmapPosition[1] + 0.3);	
			glVertex3f(-1.0f,-1.0f, 1.0f);
			
			glMultiTexCoord2f(GL_TEXTURE0, 0.0, 1.0);
			glMultiTexCoord2f(GL_TEXTURE2, lightmapPosition[0] + 0.3, lightmapPosition[1] + 0.7);
			glVertex3f(-1.0f,-1.0f,-1.0f);
			
			glMultiTexCoord2f(GL_TEXTURE0, 1.0, 1.0);
			glMultiTexCoord2f(GL_TEXTURE2, lightmapPosition[0] + 0.7, lightmapPosition[1] + 0.7);
			glVertex3f( 1.0f,-1.0f,-1.0f);
		glEnd();

		glActiveTexture(GL_TEXTURE0);
			glDisable(GL_TEXTURE_2D);
		glActiveTexture(GL_TEXTURE2);
			glDisable(GL_TEXTURE_2D);
				
		materialMode(ROOM_OBJECT);
			glActiveTexture(GL_TEXTURE0);

		if(roomTextureMode)
		{
			glEnable(GL_TEXTURE_2D);
		}

		glBindTexture(GL_TEXTURE_2D, texName[4]);
		if (roomEnv == 0)
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		else
			glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		glBegin(GL_QUADS);
			glColor3f(0.5, 0.5, 0.5);
			glNormal3d(0, 1, 0);			// Top
			glTexCoord2f(1.0, 0.0);		glVertex3f( 1.0f, 1.0f,-1.0f);
			glTexCoord2f(0.0, 0.0);		glVertex3f(-1.0f, 1.0f,-1.0f);
			glTexCoord2f(0.0, 1.0);		glVertex3f(-1.0f, 1.0f, 1.0f);
			glTexCoord2f(1.0, 1.0);		glVertex3f( 1.0f, 1.0f, 1.0f);
		glEnd();

		if(!roomTextureMode)
		{
			glDisable(GL_TEXTURE_2D);
		}
		
		glBindTexture(GL_TEXTURE_2D, texName[5]);

		glBegin(GL_QUADS);
			glNormal3d(0, 0, 1);			// Front
			glTexCoord2f(0.0, 1.0);     glVertex3f( 1.0f, 1.0f, 1.0f);
			glTexCoord2f(1.0, 1.0);     glVertex3f(-1.0f, 1.0f, 1.0f);
			glTexCoord2f(1.0, 0.0);     glVertex3f(-1.0f,-1.0f, 1.0f);
			glTexCoord2f(0.0, 0.0);     glVertex3f( 1.0f,-1.0f, 1.0f);
		
			glNormal3d(0, 0, -1);			// Back
			glTexCoord2f(1.0, 0.0);		glVertex3f( 1.0f,-1.0f,-1.0f);
			glTexCoord2f(0.0, 0.0);		glVertex3f(-1.0f,-1.0f,-1.0f);
			glTexCoord2f(0.0, 1.0);		glVertex3f(-1.0f, 1.0f,-1.0f);
			glTexCoord2f(1.0, 1.0);		glVertex3f( 1.0f, 1.0f,-1.0f);

			glNormal3d(-1, 0, 0);			// Left
			glTexCoord2f(1.0, 1.0);     glVertex3f(-1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0, 1.0);     glVertex3f(-1.0f, 1.0f,-1.0f);
			glTexCoord2f(0.0, 0.0);     glVertex3f(-1.0f,-1.0f,-1.0f);
			glTexCoord2f(1.0, 0.0);     glVertex3f(-1.0f,-1.0f, 1.0f);
			
			glNormal3d(1, 0, 0);			// Right
			glTexCoord2f(1.0, 1.0);     glVertex3f( 1.0f, 1.0f,-1.0f);
			glTexCoord2f(0.0, 1.0);     glVertex3f( 1.0f, 1.0f, 1.0f);
			glTexCoord2f(0.0, 0.0);     glVertex3f( 1.0f,-1.0f, 1.0f);
			glTexCoord2f(1.0, 0.0);     glVertex3f( 1.0f,-1.0f,-1.0f);
		glEnd();
		
		glDisable(GL_TEXTURE_2D);
	glPopMatrix();
	
	drawCone();
	drawDragonfly();
}

/**
*	Builds the cubemap based on the center of the sphere.
*/
void makeCubeMap(void)
{
	static GLfloat face[6][6] = {	{1.0, 0.0, 0.0, 0.0, -1.0, 0.0},		// +X
									{-1.0, 0.0, 0.0, 0.0, -1.0, 0.0},		// -X
									{0.0, 1.0, 0.0, 0.0, 0.0, 1.0},			// +Y
									{0.0, -1.0, 0.0, 0.0, 0.0, -1.0},		// -Y
									{0.0, 0.0, 1.0, 0.0, -1.0, 0.0},		// +Z
									{0.0, 0.0, -1.0, 0.0, -1.0, 0.0} };		// -Z
	int tx, ty, tw, th;		// GLUI viewport size
	
	glReadBuffer(GL_BACK);

	glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90.0, 1.0, 5.0, 200);
		glViewport(0, 0, CUBEMAP_SIZE, CUBEMAP_SIZE);
	
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
		for (int i = 0; i < 6; i++)
		{
			glPushMatrix();
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
				glEnable(GL_DEPTH_TEST);
				glDepthMask(GL_TRUE);
				glEnable(GL_LIGHTING);

				glLoadIdentity();
				gluLookAt(0.0, 0.0, 0.0, face[i][0], face[i][1], face[i][2], face[i][3], face[i][4], face[i][5]);

				DrawRoom();
				glFinish();
				
				glEnable(GL_TEXTURE_CUBE_MAP_ARB);
					glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubeMap);
					glCopyTexSubImage2D(cubeFaces[i], 0, 0, 0, 0, 0, CUBEMAP_SIZE, CUBEMAP_SIZE);
				glDisable(GL_TEXTURE_CUBE_MAP_ARB);
		//		glutSwapBuffers();
		//		sleep(1);
			glPopMatrix();
		}
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(70.0, 1.0, cameraDistance() > 10.0?cameraDistance() - 10.0:0.1, 200.0);
	
	GLUI_Master.get_viewport_area(&tx, &ty, &tw, &th);
	glViewport(tx, ty, tw, th);

	glMatrixMode(GL_MODELVIEW);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		glLoadIdentity();
		
	glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);	
	gluLookAt(eye[0], eye[1], eye[2], lookat[0], lookat[1], lookat[2], 0, 1, 0);
}

/**
*	Calculates the distance between the camera position
*	and the camera lookat point.  Used for gluPerspective().
*
*	@return		the distance calculated
*/
float cameraDistance(void)
{
	float dx, dy, dz;
	
	dx = eye[0] - lookat[0];
	dy = eye[1] - lookat[1];
	dz = eye[2] - lookat[2];

	return 	sqrt(dx * dx + dy * dy + dz * dz);
}

/**
*	The display callback for the GLUT display loop.
*	Draws the scene, and adds the reflective sphere.
*	After the rendering buffers are swapped, a call
*	to makeCubeMap() refreshes the cubemap, but never
*	draws that directly to the screen.
*/
void Display(void)
{
	diffuseLight[0] = diffuseLight[1] = diffuseLight[2] = lightIntensity;
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(70.0, 1.0, cameraDistance() > 10.0?cameraDistance() - 15.0:0.1, 200.0);

	glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glLightfv(GL_LIGHT0, GL_POSITION, lightPos);
		glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);	
		gluLookAt(eye[0], eye[1], eye[2], lookat[0], lookat[1], lookat[2], 0, 1, 0);

	DrawRoom();

	DrawSphere();
	
	glFlush();
	glutSwapBuffers();

	makeCubeMap();
}

/**
*	Keyboard callback function for GLUT.  Allows the user
*	to rotate the cone by pressing the 'r' key.
*
*	@param	key		the key pressed
*	@param	x		x? - unused
*	@param	y		y? - unused
*/
void keyboard(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'r': case 'R':
			rotate += 15.0;
			break;
	}
	glutPostRedisplay();
}

/**
*	The GLUI callback for GLUI buttons, etc.
*	Resets the camera, and resets the lightmap position.
*
*	@param	control		the GLUI state to control
*/
void glui_cb(int control)
{
	switch(control)
	{
		case 1:
			eye[0] = 0;
			eye[1] = 0;
			eye[2] = -40;
			lookat[0] = 0;
			lookat[1] = -5;
			lookat[2] = 0;
			break;
		case 2:
			lightmapPosition[0] = lightmapPosition[1] = 0.0;
			break;
	}
}

/**
*	The main function.  Sets up the GLUT callback functions,
*	and builds the GLUI GUI interface.
*/
int main(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT); 
    glutInitWindowPosition (100, 100);
    
	main_window = glutCreateWindow ("Room With Cone, Reflecting Sphere, and Dragonfly");
	
	Init();
	
    glutReshapeFunc(myGlutReshape);
	
    glutDisplayFunc(Display); 
	glutKeyboardFunc(keyboard);
	glutMouseFunc(myGlutMouse);
	glutMotionFunc(myGlutMotion);
	
	glui = GLUI_Master.create_glui_subwindow(main_window, GLUI_SUBWINDOW_LEFT);
	glui->add_button("Quit", 0, (GLUI_Update_CB)exit);
	glui->add_statictext("");
	
	glui->add_button("Reset Camera", 1, glui_cb);
	glui->add_statictext("");
	
	lightRollout = glui->add_rollout("Light Control");
	GLUI_Panel *lightPanel = glui->add_panel_to_panel(lightRollout, "", GLUI_PANEL_NONE);
	GLUI_Spinner *lightSpin = glui->add_spinner_to_panel(lightRollout, "Light Intensity", GLUI_SPINNER_FLOAT, &lightIntensity);
	lightSpin->set_float_limits(0.0, 5.0);
	lightmapTrans = glui->add_translation_to_panel(lightPanel, "Lightmap Position", GLUI_TRANSLATION_XY, lightmapPosition);
	lightmapTrans->scale_factor = 0.01f;
	glui->add_button_to_panel(lightPanel, "Reset Lightmap", 2, glui_cb);

	coneRollout = glui->add_rollout("Texture Control");
	GLUI_Panel *cone_panel = glui->add_panel_to_panel(coneRollout, "", GLUI_PANEL_NONE);
	glui->add_statictext_to_panel(cone_panel, "Select Cone Texture Function");
	coneTextureRadio = glui->add_radiogroup_to_panel(cone_panel, &coneMode);
	glui->add_radiobutton_to_group(coneTextureRadio, "Decal");
	glui->add_radiobutton_to_group(coneTextureRadio, "Replace");
	glui->add_radiobutton_to_group(coneTextureRadio, "Modulate");
	glui->add_radiobutton_to_group(coneTextureRadio, "Blend");
	glui->add_statictext_to_panel(cone_panel, "");

	glui->add_statictext_to_panel(cone_panel, "Select Room Texture");
	roomTextureRadio = glui->add_radiogroup_to_panel(cone_panel, &roomTextureMode);
	glui->add_radiobutton_to_group(roomTextureRadio, "Just Floor");
	glui->add_radiobutton_to_group(roomTextureRadio, "Entire Room");
	glui->add_statictext_to_panel(cone_panel, "");
	
	glui->add_statictext_to_panel(cone_panel, "Select Room Environment Function");
	roomEnvironmentRadio = glui->add_radiogroup_to_panel(cone_panel, &roomEnv);
	glui->add_radiobutton_to_group(roomEnvironmentRadio, "Modulate Room Texture");
	glui->add_radiobutton_to_group(roomEnvironmentRadio, "Replace Room Texture");
	
	
	glui->add_column_to_panel(cone_panel, false);
	glui->add_statictext("");
		
	eye[0] = 0;
	eye[1] = 0;
	eye[2] = -40;
	lookat[0] = 0;
	lookat[1] = -5;
	lookat[2] = 0;

    glutMainLoop();
    
    return 0;
}
